'''
/***************************************************************************
Name	   :  database_utilities.py
Description:  Database utility classes for FEWSTools plugin,
              updated from QGIS2
copyright  :  (C) 2019-2020 by FEWS
email      :  cholen@contractor.usgs.gov
Modified   :  12/16/2020 - cholen - Added query_db_columns
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
'''
import sqlite3

from . import fews_tools_config


def query_db(sql, params=None, multi=False):
    '''
    Run a sql query ( for SELECT )

    :param string: Input query
    :param string: Parameters for input query
    :param boolean: Expect multilple records returned?
    :return list: Query result
    '''
    db_conn = sqlite3.connect(fews_tools_config.DB_FILE)
    cursor_map = db_conn.cursor()
    if params:
        cursor_map.execute(sql, params)
    else:
        cursor_map.execute(sql)
    if multi is False:
        fetch = cursor_map.fetchone()
    else:
        fetch = cursor_map.fetchall()
    db_conn.close()
    return fetch


def query_db_columns(sql):
    '''
    Run a sql query ( for SELECT )

    :param string: Input query
    :param string: Parameters for input query
    :param boolean: Expect multilple records returned?
    :return list: Query result
    '''
    db_conn = sqlite3.connect(fews_tools_config.DB_FILE)
    cursor_map = db_conn.cursor()
    cursor_map.execute(sql)
    col_names = [description[0] for description in cursor_map.description]
    db_conn.close()
    return col_names


def write_db(sql, params=None):
    '''
    Write to db using sql stmt

    :param string: Input stmt
    :param string: Parameters
    '''
    db_conn = sqlite3.connect(fews_tools_config.DB_FILE)
    cursor_map = db_conn.cursor()
    if not params:
        cursor_map.execute(sql)
    else:
        cursor_map.execute(sql, params)
    db_conn.commit()
    db_conn.close()
