'''
/***************************************************************************
 FEWSTools
                                 A QGIS plugin
 FEWS Tools for GeoProcessing
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
    begin                : 2019-09-15
    git sha              : $Format:%H$
    copyright            : (C) 2019-2021 by FEWS
    email                : cholen@contractor.usgs.gov
    Modified:  04/14/2020 cholen - Updated order of toolbar
               05/19/2020 msun - Adjusted default workspace creation
               08/14/2020 cholen - Removed dead code, adjust tool tip
               08/29/2020 cholen - Remove set defaults tool
               10/22/2020 cholen - Change setup var name to match other files
               01/07/2021 msun - Add default output directory controller
               02/01/2021 cholen - Comment NetCDF until external tool works
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
'''
import os.path

from PyQt5.QtCore import QSettings, QTranslator, qVersion, QCoreApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QAction, QMenu, QMessageBox, QToolBar

# Initialize Qt resources from file resources.py
from .resources import *

from .basiics_controller import BasiicsController
from .climate_data_download_controller import ClimateDataDownloadController
from .changes_in_averages_controller import ChangesInAveragesController
from .contour_controller import ContourController
from .composites_controller import CompositesController
from .datasets_controller import DatasetsController
from .region_controller import RegionController
from .climatological_analysis_controller import ClimatologicalAnalysisController

from .import_archive_controller import ImportArchiveController
from .view_available_data_controller import ViewAvailableDataController
from .workspace_setup_controller import WorkspaceSetupController
from .workspace_setup_model import WorkspaceSetupModel
from .rainfall_summary_controller import RainfallSummariesController
from .extract_stats_controller import ExtractStatsController
from .reclassify_grids_controller import ReclassifyGridsController
from .help_controller import HelpController
# from .netCDF_controller import NetCDFController
from .output_controller import OutputController
# for any startup updates needed
from .datasets_model import DatasetsModel


class FEWSTools:
    '''QGIS Plugin Implementation.'''

    def __init__(self, iface):
        '''Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        '''
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'FEWSTools_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&FEWS Tools')

        # Check if plugin was started the first time in current QGIS session
        # Must be set in initGui() to survive plugin reloads
        self.first_start = None
        # define actions for Main menu drop down
        plugin_dir = os.path.dirname(__file__)
        icon_dir = os.path.join(plugin_dir, 'icons')
        self.action_geoclim = QMenu(u'GeoCLIM', self.iface.mainWindow())
        self.action_geoclim_sub = QAction(
            u'GeoCLIM Tools', self.iface.mainWindow())
        self.action_geowrsi = QAction(u'GeoWRSI', self.iface.mainWindow())
        self.action_geocof = QAction(u'GeoCOF', self.iface.mainWindow())
        self.action_reclass = QAction(
            u'Reclassify Grids', self.iface.mainWindow())
        # setup icons for toolbar
        icon_output_settings = os.path.join(icon_dir, 'output.png')
        icon_workspace_settings = os.path.join(icon_dir, 'workspace.png')
        icon_datasets = os.path.join(icon_dir, 'database.png')
        icon_regions = os.path.join(icon_dir, 'region.png')
        icon_import = os.path.join(icon_dir, 'import.png')
        icon_download = os.path.join(icon_dir, 'download.png')
        icon_available_data = os.path.join(icon_dir, 'view.png')
        icon_statistics = os.path.join(icon_dir, 'statistics.png')
        icon_averages = os.path.join(icon_dir, 'averages.png')
        icon_composites = os.path.join(icon_dir, 'composites.png')
        icon_basiics = os.path.join(icon_dir, 'basiics.png')
        icon_rainfall_summary = os.path.join(icon_dir, 'summary.png')
        icon_contour = os.path.join(icon_dir, 'contours.png')
        # icon_netCDF = os.path.join(icon_dir, 'netcdf.png')
        icon_climatological_analysis = \
            os.path.join(icon_dir, 'climAnalysis.png')
        icon_help = os.path.join(icon_dir, 'help.png')
        # setup actions for toolbar icons
        self.action_workspace = QAction(QIcon(icon_workspace_settings),
                                        'Initial Setup/Workspace Settings',
                                        self.iface.mainWindow())
        self.action_output = QAction(QIcon(icon_output_settings),
                                     'Output Options')
        self.action_datasets = QAction(QIcon(icon_datasets),
                                       'Add/Edit Datasets',
                                       self.iface.mainWindow())
        self.action_regions = QAction(QIcon(icon_regions),
                                      'Add/Edit Region',
                                      self.iface.mainWindow())
        self.action_import = QAction(QIcon(icon_import),
                                     'Import Climate Data Archives',
                                     self.iface.mainWindow())
        self.action_download = QAction(QIcon(icon_download),
                                       'Download Data by Date',
                                       self.iface.mainWindow())
        self.action_view_data = QAction(QIcon(icon_available_data),
                                        'View Available Data',
                                        self.iface.mainWindow())
        self.action_climatological_analysis = QAction(
            QIcon(icon_climatological_analysis),
            'Climatological Analysis',
            self.iface.mainWindow())
        self.action_rainfall_summary = QAction(
            QIcon(icon_rainfall_summary),
            'View/Explore Rainfall Summaries',
            self.iface.mainWindow())
        self.action_composites = QAction(QIcon(icon_composites),
                                         'Composites',
                                         self.iface.mainWindow())
        self.action_contours = QAction(QIcon(icon_contour),
                                       'Make Contours',
                                       self.iface.mainWindow())

        self.action_changes_in_averages = QAction(
            QIcon(icon_averages),
            'Calculate Long-Term changes in Averages',
            self.iface.mainWindow())
        self.action_basiics = QAction(QIcon(icon_basiics),
                                      'BASIICS - Background-Assisted Station '
                                      'Interpolation for '
                                      'Improved Climate Surfaces',
                                      self.iface.mainWindow())
        self.action_extract_stats = QAction(QIcon(icon_statistics),
                                            'Extract Statistics from Raster '
                                            'Data using Shapefile',
                                            self.iface.mainWindow())
#        self.action_NetCDF = QAction(QIcon(icon_netCDF),
#                                     'NetCDF',
#                                     self.iface.mainWindow())
        self.action_help = QAction(QIcon(icon_help),
                                   'Help',
                                   self.iface.mainWindow())

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        '''Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        '''
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('FEWSTools', message)

    def add_action(self, icon_path, text, callback,
                   enabled_flag=True, add_to_menu=True,
                   add_to_toolbar=True, status_tip=None,
                   whats_this=None, parent=None):
        '''Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        '''

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Adds plugin icon to Plugins toolbar
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToMenu(self.menu, action)
        self.actions.append(action)

        return action

    def initGui(self):
        '''
          Intialize the QGIS GUI for the plugin
        '''
        # define triggers for actions for Main menu drop down
        self.action_geoclim.pyqtConfigure(triggered=self.geoclim)
        self.action_geowrsi.pyqtConfigure(triggered=self.geowrsi)
        self.action_geocof.pyqtConfigure(triggered=self.geocof)
        self.action_reclass.pyqtConfigure(triggered=self.reclass_grids)

        # add a 'FEWS Tools' item to the main menu
        self.menu = QMenu('FEWS Tools')
        self.menu.addMenu(self.action_geoclim)
        self.menu.addAction(self.action_geowrsi)
        self.menu.addAction(self.action_geocof)
        self.action_geoclim.addAction(self.action_geoclim_sub)
        self.action_geoclim.addAction(self.action_reclass)
        self.iface.mainWindow().menuBar().insertMenu(
            self.iface.firstRightStandardMenu().menuAction(),
            self.menu)

        # users have the option to move the workspace from the default
        # so before deploying the workspace to the default location,
        # c:\Users\<user>\Documents, check if there is a workspace setting
        # in the database and if there is a GeoCLIM folder at that location
        # If so, do nothing

        # Note that if the workspace setting is 'c:\Workspace' (default value
        # in the db for a fresh deployment), and there is a GeoCLIM folder
        # at that location, do nothing.  The user may have actually
        # selected that location, so if it's there, leave it alone.

        # Else if the workspace setting is 'c:\Workspace' (default value in
        # the db for a fresh deployment), and there is NOT a GeoCLIM folder
        # at that location, deploy a copy of the workspace template to the
        # default location.
        wrksp_setup = WorkspaceSetupModel()
        settings = WorkspaceSetupController()
        workspace = wrksp_setup.get_workspace()
        folder_list = workspace.split(os.sep)
        # if folder_list has a last element that is empty, drop it...
        if folder_list[-1] == '':
            folder_list = folder_list[0:-1]
        # Check for the 'GeoCLIM' folder, because the 'Output' sub-folder
        # can also be moved to another location using the output options.
        if os.path.isdir(workspace) and folder_list[-1] == 'GeoCLIM':
            QMessageBox.information(
                self.iface.mainWindow(),
                'Existing GeoCLIM workspace detected here:',
                workspace, QMessageBox.Ok)
        else:
            settings.build_default_workspace()

    def unload(self):
        '''
        Removes the plugin menu item and icon from QGIS GUI.
        '''
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&FEWS Tools'),
                action)
            self.iface.removeToolBarIcon(action)

    def geoclim(self):
        '''
        Enable GeoCLIM functionality
        '''
        # hide all QGIS default toolbars (user can reselect, if desired)
        self.iface.digitizeToolBar().setVisible(False)
        self.iface.mapNavToolToolBar().setVisible(False)
        self.iface.advancedDigitizeToolBar().setVisible(False)
        self.iface.pluginToolBar().setVisible(False)
        self.iface.layerToolBar().setVisible(False)
        self.iface.databaseToolBar().setVisible(False)
        self.iface.fileToolBar().setVisible(False)
        self.iface.helpToolBar().setVisible(False)
        self.iface.vectorToolBar().setVisible(False)
        self.iface.rasterToolBar().setVisible(False)
        self.iface.webToolBar().setVisible(False)
        self.iface.attributesToolBar().setVisible(False)
        self.iface.mainWindow().findChild(
            QToolBar, 'mLabelToolBar').setVisible(False)
        self.iface.mainWindow().findChild(
            QToolBar, 'mLayerToolBar').setVisible(False)

        # enable and fill in the GeoCLIM toolbar
        self.geoclim_toolbar = self.iface.addToolBar('FEWS Tools')
        # connect actions to methods
        self.action_workspace.triggered.connect(self.workspace_settings)
        self.action_output.triggered.connect(self.output_settings)
        self.action_datasets.triggered.connect(self.datasets)
        self.action_regions.triggered.connect(self.regions)
        self.action_import.triggered.connect(self.import_archives)
        self.action_download.triggered.connect(self.download_data)
        self.action_view_data.triggered.connect(self.view_data)
        self.action_extract_stats.triggered.connect(self.extract_stats)
        self.action_changes_in_averages.triggered.connect(
            self.changes_averages)
        self.action_basiics.triggered.connect(self.run_basiics)
        self.action_composites.triggered.connect(self.composites)
        self.action_contours.triggered.connect(self.contours)
        self.action_rainfall_summary.triggered.connect(self.rainfall_summary)
        self.action_climatological_analysis.triggered.connect(
            self.climatological_analysis)

        help_controller = HelpController(self.iface)
        self.action_help.setMenu(help_controller.get_menu())
#        netCDF_controller = NetCDFController(self.iface)
#        self.action_NetCDF.setMenu(netCDF_controller.get_menu())

        # add actions to toolbar
        self.geoclim_toolbar.addAction(self.action_workspace)
        self.geoclim_toolbar.addAction(self.action_output)
        self.geoclim_toolbar.addAction(self.action_datasets)
        self.geoclim_toolbar.addAction(self.action_regions)
        self.geoclim_toolbar.addAction(self.action_import)
        self.geoclim_toolbar.addAction(self.action_download)
        self.geoclim_toolbar.addAction(self.action_view_data)
        self.geoclim_toolbar.addAction(self.action_climatological_analysis)
        self.geoclim_toolbar.addAction(self.action_rainfall_summary)
        self.geoclim_toolbar.addAction(self.action_composites)
        self.geoclim_toolbar.addAction(self.action_contours)
        self.geoclim_toolbar.addAction(self.action_changes_in_averages)
        self.geoclim_toolbar.addAction(self.action_basiics)
        self.geoclim_toolbar.addAction(self.action_extract_stats)
        # self.geoclim_toolbar.addAction(self.action_NetCDF)
        self.geoclim_toolbar.addAction(self.action_help)

        DatasetsModel().database_startup_updates()

    def geowrsi(self):
        '''
        Enable GeoWRSI functionality
        '''
        QMessageBox.information(self.iface.mainWindow(),
                                'Not Implemented!!',
                                'The GeoWRSI functionality is not '
                                'implemented yet. Please select '
                                'another tool.',
                                QMessageBox.Ok)

    def geocof(self):
        '''
        Enable GeoCOF functionality
        '''
        QMessageBox.information(self.iface.mainWindow(),
                                'Not Implemented!!',
                                'The GeoCOF functionality is not '
                                'implemented yet. Please select '
                                'another tool.',
                                QMessageBox.Ok)

    def reclass_grids(self):
        '''
        Enable Reclassify Grids Functionality
        '''
        reclass_dlg = ReclassifyGridsController()
        reclass_dlg.exec_()

    def workspace_settings(self):
        '''
        Workspace settings functionality
        '''
        wrk_dlg = WorkspaceSetupController()
        wrk_dlg.exec_()

    def output_settings(self):
        '''
        Output directory settings functionality
        '''
        output_dir_dlg = OutputController()
        output_dir_dlg.exec_()

    def datasets(self):
        '''
        Datasets functionality
        '''
        dataset_dlg = DatasetsController()
        dataset_dlg.exec_()

    def regions(self):
        '''
        Regions functionality
        '''
        region_dlg = RegionController()
        region_dlg.exec_()

    def import_archives(self):
        '''
        Import archives functionality
        '''
        archive_dlg = ImportArchiveController()
        archive_dlg.exec_()

    def composites(self):
        '''
        Composites functionality
        '''
        composite_dlg = CompositesController()
        composite_dlg.exec_()

    def download_data(self):
        '''
        Download data functionality
        '''
        download_data_dlg = ClimateDataDownloadController()
        download_data_dlg.exec_()

    def view_data(self):
        '''
        View data functionality
        '''
        view_dlg = ViewAvailableDataController()
        view_dlg.exec_()

    def changes_averages(self):
        '''
        BASIICS functionality
        '''
        averages_dlg = ChangesInAveragesController()
        averages_dlg.exec_()

    def extract_stats(self):
        '''
        Extract stats functionality
        '''
        extract_stats_dlg = ExtractStatsController()
        extract_stats_dlg.exec_()

    def run_basiics(self):
        '''
        BASIICS functionality
        '''
        basiics_dlg = BasiicsController()
        basiics_dlg.exec_()

    def contours(self):
        '''
         Contours functionality
        '''
        contour_dlg = ContourController()
        contour_dlg.exec_()

    def rainfall_summary(self):
        '''
        Rainfall summary functionality
        '''
        rainfall_summary_dlg = RainfallSummariesController()
        rainfall_summary_dlg.exec_()

    def climatological_analysis(self):
        '''
        Climatological analysis functionality
        '''
        climatological_analysis_dialog = ClimatologicalAnalysisController()
        climatological_analysis_dialog.exec_()
