"""
/***************************************************************************
Name            :  help_controller.py
Description     :  A class for help icon
copyright       :  (C) 2019 by FEWS
email           :  minxuansun@contractor.usgs.gov
Created         :  3/23/2020 Minxuan Sun
Modified        :  MM/DD/YYYY - <developer> - Description
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt5.QtWidgets import QMenu, QAction
from .Ui_help_about import Ui_HelpAbout
from PyQt5.QtWidgets import QDialog

import os


class HelpController:
    def __init__(self, iface):
        """
        Constructor - Add menu to the Help icon
        """
        self.menu = QMenu()
        action_about = \
            QAction('About', iface.mainWindow())
        action_view_geoclim_manual = \
            QAction('View GeoCLIM Manual', iface.mainWindow())
        action_about.triggered.connect(self.about)
        action_view_geoclim_manual.triggered.connect(self.view_geoclim_manual)
        self.menu.addAction(action_about)
        self.menu.addAction(action_view_geoclim_manual)

    @staticmethod
    def about():
        """
        Static method used to open a image to introduce GeoClIM
        """
        about_image_path = \
            os.path.join(os.path.dirname(__file__), 'about.png')

        class HelpAboutController(QDialog):
            def __init__(self):
                QDialog.__init__(self)
                self.ui = Ui_HelpAbout(about_image_path)
                self.ui.setupUi(self)

        dlg = HelpAboutController()
        dlg.exec_()

    @staticmethod
    def view_geoclim_manual():
        """
        Static method used to open the manual
        :return:
        """
        os.startfile(
            os.path.join(os.path.dirname(__file__), 'geoclim-manual.pdf'))

    def get_menu(self):
        return self.menu
